//	 General purpose routines for HCS08 processors
//
//  Copyright 2009 Filip Mulier
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

/* MODULE general */
 #include "general.h"
 
 /* Include shared modules, which are used for whole project */
#include "derivative.h"

/*Function used to calculate low pass Bessel filter (matched Z transform)*/

// Uses the equation instore_new=in*fparam/256 + instore*(1-fparam/256)
// parameter fparam can be calculated at the site:
// http://www-users.cs.york.ac.uk/~fisher/mkfilter/trad.html


//Uses the byte *lsb_store for extra precision when filtering



fixed lowpass(fixed in,fixed instore, byte fparam, byte *lsb_store)
  {
   unsigned char signstore1,signstore2,lsba,lsb2,lsb;
   
   lsb=*lsb_store;
   
   asm
    {
        ;Convert the input to unsigned and keep track of sign     
            clr signstore1          ;clear sign storage
            tst in:0								 ;check if msb sign is negative
            bge mult1      ;check multiplier sign bit and negate
            com in:0
            neg in:1
            bne mprsign
            inc in:0
mprsign:    inc signstore1                ;set sign bit for negative number


       ;Multiply (in:in+1) by fparam   
mult1:    ldx in:1							;load x-reg w/multiplier lsb
          lda fparam							;load acc w/multiplicand msb
          mul									;multiply
          stx in:1							;save carry from multiply
          sta lsba							;store lsb of interm. result 2.
          ldx in:0							;load x-reg w/multiplier msb
          lda fparam							;load acc w/multiplicand msb
          mul									;multiply
          add in:1							;add carry from previous multiply
          sta in:1             ;store 2nd byte of interm. resutt 2 in t6.
          bcc noincb1					;check for carry from addition
          incx								;increment msb of interm. result 2.
noincb1: stx in:0				;store msb of interm. result 2.

       ;Convert in result back to signed
          lda signstore1				 ;load sign bit
          cmp #1 							 ;check for negative
          bne next1           ;branch if not negative
          com in:0             ;ones complement msb
          com in:1             ;ones complement result byte 2
          neg lsba          ;ones complement result lsb
          bne next1             ; load lsb of result
          inc in:1
          bne next1
          inc in:0

next1:          com fparam  ;calculate 255-fparam
                inc fparam  ; calculate 256-fparam

        ;Convert the instore to unsigned and keep track of sign     
            clr signstore2          ;clear sign storage
            tst instore:0								 ;check if msb sign is negative
            bge mult2      ;check multiplier sign bit and negate
            com      instore:0
            com      instore:1
            neg      lsb
            bne      lneg0
            inc      instore:1
            BNE      lneg0
            inc      instore:0
lneg0:      inc signstore2
        


       ;Multiply (instore:instore+1,lsb) by 255-fparam 
mult2:    ldx lsb							;load x-reg w/multiplier lsb
          lda fparam							;load acc w/multiplicand msb
          mul
          stx lsb
          sta lsb2
          ldx instore:1							;load x-reg w/multiplier lsb
          lda fparam							;load acc w/multiplicand msb
          mul									;multiply
          add lsb
          sta lsb
          bcc noincb3
          inx
noincb3:  stx instore:1
          ldx instore:0							;load x-reg w/multiplier msb
          lda fparam							;load acc w/multiplicand msb
          mul									;multiply
          add instore:1							;add carry from previous multiply
          sta instore:1             ;store 2nd byte of interm. resutt 2 in t6.
          bcc noincb2					;check for carry from addition
          incx								;increment msb of interm. result 2.
noincb2: stx instore:0				;store msb of interm. result 2.
          
          
          
          
;;only use the 3 most significant bytes since we need to divide by 256


       ;Convert in result back to signed
          lda signstore2				 ;load sign bit
          cmp #1 							 ;check for negative
          bne next2           ;branch if not negative
            com      instore:0
            com      instore:1
            neg      lsb
            bne      next2
            inc      instore:1
            BNE      next2
            inc      instore:0

      ; Add the results together
next2:    lda lsba
          add lsb
          sta lsb            
          lda in:1
          adc instore:1
          sta instore:1
          lda in:0
          adc instore:0
          sta instore:0
          


      
      
    } 
  *lsb_store=lsb;
  
  return(instore);  
    
  }



/*Routine for fixed point multiplication.*/
/*Result is done with 32 bits and then reduced*/
/*Minimal stack usage*/
fixed fmul(fixed x,fixed y)
  {
    /*unsigned int  res12;  */
		unsigned char t2,t5,t6,signstore;
		/*result is stored in x*/
  
    asm
      {
            
       ;Convert the input to unsigned and keep track of sign     
            clr signstore          ;clear sign storage
            tst x:0								 ;check if msb sign is negative
            bge tst2      ;check multiplier sign bit and negate
                                    ;(two's complement) if set
            neg x:1           ;two's comp multiplier lsb
            bcc nosub1              ;check for borrow from zero
            neg x:0             ;two's comp multiplier msb
            dec x:0             ;decrement msb for borrow
            bra mprsign             ;finished
nosub1:     neg x:0             ;two's comp multiplier msb (no borrow)
mprsign:    inc signstore                ;set sign bit for negative number
tst2:       tst y:0                 ;check if msb sign is negative
            bge mltsub    ;check multiplicand sign bit and negate
                                        ;(two's complement) if set
            neg y:1           ;two's comp multiplicand lsb
            bcc nosub2              ;check for borrow from zero
            neg y:0             ;two's comp multiplicand msb
            dec y:0             ;decrement msb for borrow
            bra mpcsign             ;finished
nosub2:     neg y:0             ;two's comp multiplicand msb (no borrow)
mpcsign:    inc signstore                ;set or clear sign bit      
      
      
      
      
      
      ;Multiply (x:x+1) by y+1
mltsub:     ldx x:1	            ;load x-reg w/multiplier lsb
          lda y:1	            ;load acc w/multiplicand lsb
          mul									;multiply
          stx t6							;save carry from multiply
     ;    sta res3						;normally store lsb of final result,
                              ; but for fixed point don't need it
          ldx x:0							;load x-reg w/multiplier msb
          lda y:1							;load acc w/multiplicand lsb
          mul									;multiply
          add t6							;add carry from previous multiply
          sta t2							;store 2nd byte of interm. result
          bcc noinca					;check for carry from addition
          incx								;increment msb of interm. result 1.
      ; noinca: stx t1					;store msb of interm. result 1.
        noinca: stx y:1       ;store msb of interem result 1 in y:1
          clr t6							;clear storage for carry
       
       ;Multiply (x:x+1) by y   
          ldx x:1							;load x-reg w/multiplier lsb
          lda y:0							;load acc w/multiplicand msb
          mul									;multiply
          stx t6							;save carry from multiply
          sta t5							;store lsb of interm. result 2.
          ldx x:0							;load x-reg w/multiplier msb
          lda y:0							;load acc w/multiplicand msb
          mul									;multiply
          add t6							;add carry from previous multiply
         ; sta t4							;store 2nd byte of interm. result 2.
          sta y:0             ;store 2nd byte of interm. resutt 2 in y:0.
          bcc noincb					;check for carry from addition
          incx								;increment msb of interm. result 2.
  ;     noincb: stx t3				;normally store msb of interm. result 2.
                              ;but for fixed point don't need to.
    
    ;Add the intermediate results and store the remaining three bytes of the
    ;final value in locations y, y+1.

 noincb:  lda t2							;load acc with 2nd byte of 1st result
          add t5							;add acc with lsb of 2nd result
          sta x:1					;store 2nd byte of final result
       ;   lda t1							;load acc with msb of 1st result
          lda y:1             ;load acc with msb of 1st result now in y:1
       ;  adc t4							;add w/ carry 2nd byte of 2nd result
          adc y:0             ;add w/ carry 2nd byte og 2nd result noow in y:0
          sta x:0					;store 3rd byte of final result
      ;    lda t3							;normally load acc with msb from 2nd result
      ;    adc #0							;normally add any carry from previous addition
      ;    sta res0						;normally store msb of final result
                              ;but for fixed point don't need to
     
     ;Some tricky assignments are going on above.
     ;Since we are doing fixed point, we shift right by 8 bits (a byte)
     ;and return an unsigned integer res12. res0 and res3 are not used.    
     
     
     ;Convert to signed
          lda signstore				 ;load sign bit
          cmp #1 							 ;check for negative
          bne notneg           ;branch if not negative
          neg x:1          ;twos complement result lsb
          bcc nosub3           ;check for borrow from zero
          neg x:0             ;two's comp result msb
          dec x:0             ;decrement result msb for borrow
          bra notneg              ;finished
nosub3:   neg x:0             ;two's comp result msb without decrement

notneg:   nop     
      }
      return(x);
      
  }


/*Routine for fixed point division.*/
/*Result is done with 24 bits and then reduced*/
/*Minimal stack usage*/
fixed fdiv(fixed x,fixed y)
 {
    /*unsigned int  res12;  */
    
    
		unsigned char loopcount,signstore,dividend3;
		unsigned int rem;
		/*result is stored in x*/
 
    asm
      {
            
       ;Convert the input to unsigned and keep track of sign     
            clr signstore          ;clear sign storage
            tst x:0								 ;check if msb sign is negative
            bge tst2      ;check dividend sign bit and negate
                                    ;(two's complement) if set
            neg x:1           ;two's comp dividend lsb
            bcc nosub1              ;check for borrow from zero
            neg x:0             ;two's comp dividend msb
            dec x:0             ;decrement msb for borrow
            bra mprsign             ;finished
nosub1:     neg x:0             ;two's comp dividend msb (no borrow)
mprsign:    inc signstore                ;set sign bit for negative number
tst2:       tst y:0                 ;check if msb sign is negative
            bge divsub    ;check divisor sign bit and negate
                                        ;(two's complement) if set
            neg y:1           ;two's comp divisor lsb
            bcc nosub2              ;check for borrow from zero
            neg y:0             ;two's comp divisor msb
            dec y:0             ;decrement msb for borrow
            bra mpcsign             ;finished
nosub2:     neg y:0             ;two's comp divisor msb (no borrow)
mpcsign:    inc signstore                ;set or clear sign bit      



  divsub:   lda #24                ;
           sta loopcount                    ;loop counter for number of shifts
 

        clr rem:0               ;zero remainder msb
        clr rem:1             ;zero remainder lsb
        clr dividend3             ;zero lsb of dividend

;
;     Shift each byte of dividend and remainder one bit to the left
;
shftlp:  lda rem:0               ;get remainder msb
        rola                            ;shift remainder msb into carry
        ROL     dividend3              ;shift dividend lsb
        ROL     x:1              ;shift 2nd byte of dividend
        ROL     x:0              ;shift msb byte of dividend
        ROL     rem:1             ;shift remainder lsb
        ROL     rem:0               ;shift remainder msb
;
;     Subtract both bytes of the divisor from the remainder
;
        LDA     rem:1             ;get remainder lsb
        SUB     y:1                    ;subtract divisor lsb from remainder lsb
        STA     rem:1             ;store new remainder lsb
        LDA     rem:0               ;get remainder msb
        SBC     y:0                    ;subtract divisor msb from remainder msb
        STA     rem:0               ;store new remainder msb
        LDA     dividend3              ;get low byte of dividend/quotient
        SBC     #0                      ;dividend low bit holds subtract carry
        STA     dividend3              ;store low byte of dividend/quotient
;
;     Check dividend/quotient lsb. If clear, set lsb of quotient to indicate
;     successful subraction, else add both bytes of divisor back to remainder
;


                                         ;check for a carry from subtraction
                                         ;and add divisor to remainder if set
        bit #1													 ;logical and with 1
        beq setlsb											 ;branch if equal
        
        LDA     rem:1             ;get remainder lsb
        ADD     y:1                    ;add divisor lsb to remainder lsb
        STA     rem:1             ;store remainder lsb
        LDA     rem:0               ;get remainder msb
        ADC     y:0                    ;add divisor msb to remainder msb
        STA     rem:0               ;store remainder msb
        LDA     dividend3              ;get low byte of dividend
        ADC     #0                      ;add carry to low bit of dividend
        STA     dividend3              ;store low byte of dividend
        BRA     decrmt                  ;do next shift and subtract

                                        ;set lsb of quotient to indicate
                                        ;successive subtraction
setlsb: lda dividend3
        ora #1
        sta dividend3                                        
                                        
                                        
                                        
decrmt:  DBNZ    loopcount,shftlp             ;decrement loop counter and do next
                                        ;shift

;
;Move the two lsb of the resulting quotient into x
;

         lda x:1
         sta x:0
         lda dividend3
         sta x:1

     ;Convert to signed
          lda signstore				 ;load sign bit
          cmp #1 							 ;check for negative
          bne notneg           ;branch if not negative
          neg x:1          ;twos complement result lsb
          bcc nosub3           ;check for borrow from zero
          neg x:0             ;two's comp result msb
          dec x:0             ;decrement result msb for borrow
          bra notneg              ;finished
nosub3:   neg x:0             ;two's comp result msb without decrement

notneg:   nop     



        
      }
			return(x);
			
			
 }

 

/*Routines for fixed point with overflow clamping*/

#pragma INLINE
/*Routine for fixed point addition with no rollover - output is clamped*/
fixed faddc(fixed x,fixed y)
  {
   fixed res; 
    
    res=x+y;
    
    if(x>0 && y>0 && res<0)
        res=32767;      //overflow in pos
    
    if(x<0 && y<0 && res>0)
        res=-32768;     //overflow in neg    
  
  return(res);  
  }


/*Routine for fixed point multiplication.*/
/*Result is done with 32 bits and then reduced*/
/*Minimal stack usage*/
/*overflow is detected and output is clamped*/

#pragma MESSAGE DISABLE C20000
#pragma MESSAGE DISABLE C20001

fixed fmulc(fixed x,fixed y)
  {
    /*unsigned int  res12;  */
		unsigned char t2,t3,t5,t6,signstore;
		/*result is stored in x*/
  
    asm
      {
            
       ;Convert the input to unsigned and keep track of sign     
            clr signstore          ;clear sign storage
            tst x:0								 ;check if msb sign is negative
            bge tst2      ;check multiplier sign bit and negate
                                    ;(two's complement) if set
            neg x:1           ;two's comp multiplier lsb
            bcc nosub1              ;check for borrow from zero
            neg x:0             ;two's comp multiplier msb
            dec x:0             ;decrement msb for borrow
            bra mprsign             ;finished
nosub1:     neg x:0             ;two's comp multiplier msb (no borrow)
mprsign:    inc signstore                ;set sign bit for negative number
tst2:       tst y:0                 ;check if msb sign is negative
            bge mltsub    ;check multiplicand sign bit and negate
                                        ;(two's complement) if set
            neg y:1           ;two's comp multiplicand lsb
            bcc nosub2              ;check for borrow from zero
            neg y:0             ;two's comp multiplicand msb
            dec y:0             ;decrement msb for borrow
            bra mpcsign             ;finished
nosub2:     neg y:0             ;two's comp multiplicand msb (no borrow)
mpcsign:    inc signstore                ;set or clear sign bit      
      
      
      
      
      
      ;Multiply (x:x+1) by y+1
mltsub:     ldx x:1	            ;load x-reg w/multiplier lsb
          lda y:1	            ;load acc w/multiplicand lsb
          mul									;multiply
          stx t6							;save carry from multiply
     ;    sta res3						;normally store lsb of final result,
                              ; but for fixed point don't need it
          ldx x:0							;load x-reg w/multiplier msb
          lda y:1							;load acc w/multiplicand lsb
          mul									;multiply
          add t6							;add carry from previous multiply
          sta t2							;store 2nd byte of interm. result
          bcc noinca					;check for carry from addition
          incx								;increment msb of interm. result 1.
      ; noinca: stx t1					;store msb of interm. result 1.
        noinca: stx y:1       ;store msb of interem result 1 in y:1
          clr t6							;clear storage for carry
       
       ;Multiply (x:x+1) by y   
          ldx x:1							;load x-reg w/multiplier lsb
          lda y:0							;load acc w/multiplicand msb
          mul									;multiply
          stx t6							;save carry from multiply
          sta t5							;store lsb of interm. result 2.
          ldx x:0							;load x-reg w/multiplier msb
          lda y:0							;load acc w/multiplicand msb
          mul									;multiply
          add t6							;add carry from previous multiply
         ; sta t4							;store 2nd byte of interm. result 2.
          sta y:0             ;store 2nd byte of interm. resutt 2 in y:0.
          bcc noincb					;check for carry from addition
          incx								;increment msb of interm. result 2.
 noincb:  stx t3			      	;store msb of interm. result 2. to check for overflow
                              
    
    ;Add the intermediate results and store the remaining three bytes of the
    ;final value in locations y, y+1.

          lda t2							;load acc with 2nd byte of 1st result
          add t5							;add acc with lsb of 2nd result
          sta x:1					;store 2nd byte of final result
       ;   lda t1							;load acc with msb of 1st result
          lda y:1             ;load acc with msb of 1st result now in y:1
       ;  adc t4							;add w/ carry 2nd byte of 2nd result
          adc y:0             ;add w/ carry 2nd byte og 2nd result noow in y:0
          sta x:0					;store 3rd byte of final result
          lda t3							;load acc with msb from 2nd result
          adc #0							;add any carry from previous addition
          sta t3						  ;store msb of final result to later check for overflow conditions
                              
     
     ;Some tricky assignments are going on above.
     ;Since we are doing fixed point, we shift right by 8 bits (a byte)
     ;and return an unsigned integer res12. res0 is not used res3 (t3) is used for checking overflow.    
     
     ;
     
     ;Check for overflow (-32768, 32767 is the range allowed)
          lda signstore       ;load sign bit
          cmp #1              ;check for negative sign
          bne posi            ;branch if not negative
          lda t3              ;load full msb
          cmp #0
          bne negoverflow     ;if not zero, we overflowed
          lda x:0             ;check result msb 
          cmp #128            ; compare with 128 - the sign bit  (overflow if >32768)
          bhi negoverflow     ; if greater than we know its overflow
          blo inrange         ; if less than we know its not an overflow
          lda x:1             ; if equal, we need to check the lsb of the result
          cmp #0
          bne negoverflow     ; if the lsb is not zero, its overflow
          bra inrange         ; only case now is in range
posi:     lda t3              ; check for overflow for positive situation 
          cmp #0
          bne posoverflow     ; if full msb is not zero, we have overflow
          lda x:0             ; check result msb
          cmp #128            ;compare with 128, the sign bit (overflow if >32767)
          bhs posoverflow     ; overflow if >=
          bra inrange         ; pass over for in range
negoverflow: lda #0           ; clamp output to 32768 for negative overflow
          sta x:1             ; update lsb
          lda #128            ; 
          sta x:0             ;update msb
          bra inrange         ;now we are in range
posoverflow: lda #255
          sta x:1             ; clamp output to 32767 for positive overflow
          lda #127            ; 
          sta x:0             ;update msb
                              ;now we are in range
          
     
     ;Convert to signed
inrange:  lda signstore				 ;load sign bit
          cmp #1 							 ;check for negative
          bne notneg           ;branch if not negative
          neg x:1          ;twos complement result lsb
          bcc nosub3           ;check for borrow from zero
          neg x:0             ;two's comp result msb
          dec x:0             ;decrement result msb for borrow
          bra notneg              ;finished
nosub3:   neg x:0             ;two's comp result msb without decrement

notneg:   nop     
      }
      return(x);
      
  }


#pragma MESSAGE DEFAULT C20000
#pragma MESSAGE DEFAULT C20001


//arctan routines are based on a posting in comp.dsp:
//Subject: DSP Trick:  Fixed-pt. Atan2 with Self Normalization
//From: Jim Shima
//Date: 1999/04/23
//Newsgroups: comp.dsp
//THIS WORK IS PLACED IN THE PUBLIC DOMAIN
  


fixed farctan2(byte x, byte y)
{

byte yabs; 
fixed r,angle; 

 /*function computes arctan and returns an integer in the range -804 to 804 (-pi to pi)*/

/*Define byte value of zero as 128 since int is -128 to 127*/
 
 /*Compute absolute value of y*/
 
 if(y>=128)
    yabs=y-128;
 else
    yabs=128-y;
   
/*Still need to worry about divide by zero - no formula seems to take care of it*/   
   
   if (x>=128)
   {
      
      r = fdiv((x-128 - yabs), (x-128 + yabs));
//      angle = 201 - fmul(201, r);		 //201=pi*256/4
        angle=201+fmul(51,fmul(fmul(r,r),r))-fmul(250,r);		 //third-order approx
    
   }
   else
   {
      
      r = fdiv((x-128 + yabs) , (yabs - x+128));
//      angle = 603 - fmul(201, r);				 //603=3*pi*256/4
        angle=603+fmul(51,fmul(fmul(r,r),r))-fmul(250,r);    //third order approx
   }
   if (y < 128)
   return(-angle);     // negate if in quad III or IV
   else
   return(angle);
   
  
}



fixed farctan2x10(byte x, byte y)
{

byte yabs; 
fixed r,angle; 

 /*function computes arctan and returns an integer in the range -8042 to 8042 (-pi*10 to pi*10)*/

/*Define byte value of zero as 128 since int is -128 to 127*/
 
 /*Compute absolute value of y*/
 
 if(y>=128)
    yabs=y-128;
 else
    yabs=128-y;
   
/*Still need to worry about divide by zero - no formula seems to take care of it*/   
   
   if (x>=128)
   {
      
      r = fdiv((x-128 - yabs), (x-128 + yabs));
//      angle = 201 - fmul(201, r);		 //201=pi*256/4
        angle=2011+fmul(520,fmul(fmul(r,r),r))-fmul(2504,r);		 //third-order approx
    
   }
   else
   {
      
      r = fdiv((x-128 + yabs) , (yabs - x+128));
//      angle = 603 - fmul(201, r);				 //603=3*pi*256/4
        angle=6033+fmul(520,fmul(fmul(r,r),r))-fmul(2504,r);    //third order approx
   }
   if (y < 128)
   return(-angle);     // negate if in quad III or IV
   else
   return(angle);
   
  
}



fixed farctan2fs(byte x, byte y)
{

byte yabs; 
fixed r,angle; 

 /*function computes arctan and returns an integer in the range -32767 to 32767 
   in fixed point that is (-pi*40.7424214 to pi*40.7424214)*/

/*Define byte value of zero as 128 since int is -128 to 127*/
 
 /*Compute absolute value of y*/
 
 if(y>=128)
    yabs=y-128;
 else
    yabs=128-y;
   
/*Still need to worry about divide by zero - no formula seems to take care of it*/   
   
   if (x>=128)
   {
      
      r = fdiv((x-128 - yabs), (x-128 + yabs));
//      angle = 201 - fmul(201, r);		 //201=pi*256/4
//        angle=2011+fmul(520,fmul(fmul(r,r),r))-fmul(2504,r);		 //third-order approx
// pi/4*256*40.742=32767/4=8192
// .2035*256*40.7424214=2123;
// -.9783*256*40.7424214=-10204
         angle=8192+fmul(2123,fmul(fmul(r,r),r))-fmul(10204,r);		 //third-order approx
   }
   else
   {
      
      r = fdiv((x-128 + yabs) , (yabs - x+128));
//      angle = 603 - fmul(201, r);				 //603=3*pi*256/4
//        angle=6033+fmul(520,fmul(fmul(r,r),r))-fmul(2504,r);    //third order approx
// 3*pi/4*256*40.742=3*32767/4=24576
        angle=24576+fmul(2123,fmul(fmul(r,r),r))-fmul(10204,r);    //third order approx
   }
   if (y < 128)
   return(-angle);     // negate if in quad III or IV
   else
   return(angle);
   
  
}


fixed farctan2fsf(fixed x, fixed y)
{

fixed yabs; 
fixed r,angle; 

 /*function computes arctan and returns an integer in the range -32767 to 32767 
   in fixed point that is (-pi*40.7424214 to pi*40.7424214)*/

 
 /*Compute absolute value of y*/
 
 if(y>=0)
    yabs=y;
 else
    yabs=-y;
   
/*Still need to worry about divide by zero - no formula seems to take care of it*/   
   
   if (x>=0)
   {
      
      r = fdiv((x - yabs), (x + yabs));
         angle=8192+fmulc(2123,fmulc(fmulc(r,r),r))-fmulc(10204,r);		 //third-order approx
   }
   else
   {
      
      r = fdiv((x + yabs) , (yabs - x));
        angle=24576+fmulc(2123,fmulc(fmulc(r,r),r))-fmulc(10204,r);    //third order approx
   }
   if (y < 0)
   return(-angle);     // negate if in quad III or IV
   else
   return(angle);
   
  
}


fixed fsinfsf(fixed ang)
  {
  
  fixed res,ascaled,a2;
  byte quad;
    
    /*Function computes the sine of the argument. Angle is an integer in the range -32767 to 32767 
   in fixed point that is (-pi*40.7424214 to pi*40.7424214)*/
      /*output is -256 to 256*/

   
   /*figure out which quadrant the angle is in and map back to the first */
   if (ang>=0)
     if(ang<16384)
      {
       ascaled=ang;
       quad=1; 
      }
      else
      {
       ascaled=32767-ang; 
       quad=2;
      }
    else
      if (ang<-16384)
       {
       ascaled=32767+ang;
       quad=3;  
       }
       else
       {
       ascaled=-ang; 
       quad=4; 
       }
   
   /*scale the angle*/
   ascaled=fmulc(ascaled,6);
   a2=fmulc(ascaled,ascaled);
   
   /*calculate polynomial*/
   res=269-fmulc(35,a2);
   res=269-fmulc(fmulc(res,a2),24); //might be better to switch order based on scale
   res=269-fmulc(fmulc(res,a2),49); //ditto
   
   res=fmulc(res,ascaled);
   
   /*correct the sign for Q3 and Q4*/
   if(quad>2)
    res=-res;
   
   return(res);
    
  }


fixed fcosfsf(fixed ang)
  {
  
  fixed res;
    
    /*Function computes the cosine of the argument. Angle is an integer in the range -32767 to 32767 
   in fixed point that is (-pi*40.7424214 to pi*40.7424214)*/
   /*output is -256 to 256*/
  
  res=fsinfsf(16384-ang);
  
  return(res);
   
  }
  


fixed fdiv10(fixed i)         
//function does an approximate divide by 10 approximate to 19 bits
//see http://www.cs.uiowa.edu/~jones/bcd/divide.html
  {
  unsigned short q,a;
  byte sign;
  short y,r;
  
  if (i<0)
    {
    sign=1;
    a=-i;
    }
    else
    {
    sign=0;
    a=i;
    }
  
  q = ((a >> 1) + a) >> 1; /* Q = A*0.11 */
	q = ((q >> 4) + q)     ; /* Q = A*0.110011 */
	q = ((q >> 8) + q) >> 3; /* Q = A*0.00011001100110011 */
	
        /* either Q = A/10 or Q+1 = A/10 for all A < 534,890 */
	
	
	
	

// q or q+1 will be a/10

  y=q;
  r=fmul10(y);
  //check if q+1 was a/10
  r=a-r;

  if(r>=10) 
    y=y+1;
  
  if(sign)
    y=-y;
  return(y);
  }
  
  
fixed fmul10(fixed a)
//function does a multiply by 10
  {
  fixed q ;
  a=a<<1;
  q=a<<2;
  q=q+a;
  return(q);
  }

signed char interpolate(byte x)
{

/* Interpolation data x_in needs to be ordered least to greatest */
/* See excel spreadsheet in project for data calculation */

/* Bare Thermistor for testing */
/*
#define ELEMENTS 8
static const byte x_in[ELEMENTS]=			{14,20,27,47,52,82,85,100};
static const signed char y_out[ELEMENTS]=	{-40,-20,0,32,40,66,70,90};

*/

/*Siemens Protomatik Thermistor*/
/*See excel spreadsheet in project for data calculation*/
#define ELEMENTS 15
static const byte x_in[ELEMENTS]=			{37,46,55,66,78,92,106,122,137,151,164,177,188,210,225};
static const signed char y_out[ELEMENTS]=	{-40,-31,-22,-13,-4,5,14,23,32,41,50,59,68,86,104};



byte i;

if(x<=x_in[0])
	return(y_out[0]);
if(x>=x_in[ELEMENTS-1])
	return(y_out[ELEMENTS-1]);
	
i=0;
while(x>x_in[i])
	i++;
	
return( (x-x_in[i-1])*(y_out[i]-y_out[i-1])/(x_in[i]-x_in[i-1])+y_out[i-1]);
}










/*General time delay useful for bit banging I/O*/

void timedelay(unsigned char d)
{
	unsigned char	i;
	
	for(i = d; i > 0; i--){
	};
	
	
}

void Delay(unsigned int d) 
  {
  
  unsigned int i;
  for(i=d;i>0;i-- )
    {
    };
  }
  
  


/*Random number generator*/
/*initialize *idnum with any number >0*/
void ran0(long *idnum)
  {
    long k;
    
    k=(*idnum)/127773;
    *idnum=16807*(*idnum-k*127773)-2836*k;
    if(*idnum<0) 
      *idnum += 2147483647;
    
  }





/* END general */
