#include <hidef.h> /* for EnableInterrupts macro */
#include "derivative.h" /* include peripheral declarations */
#include "main_asm.h" /* interface to the assembly module */
#ifdef __cplusplus
extern "C"
#endif
void MCU_init(void); /* Device initialization function declaration */

#include "doonstack.h"
#include "general.h"

#define BUSCLK  16777216
#define vFCDIV  (BUSCLK/200000-1)


/* Initialization of the CPU registers in FLASH */

/*During reset events, the contents of NVPROT and NVOPT in the nonvolatile register area of the FLASH
memory are transferred into corresponding FPROT and FOPT working registers in the high-page registers
to control security and block protection options.*/

/* NVPROT: 7FPS    6FPS      5FPS      4FPS      3FPS      2FPS      1FPS      0FPDIS*/
/*         1         1       1         0         0         0         0         0 */  
// Use 1 bank for storing calibration info
// E000-E1FF
// Write-protect addresses ranging from 0xE200 up to 0xFFFF
// Then 0xE1FF is the last unprotected address
const unsigned char NVPROT_INIT @0x0000FFBD = (0xE1FF>>8)&0xFE;

/* NVOPT: KEYEN=0,FNORED=1,SEC01=1,SEC00=0 */
/* NVOPT:  7KEYEN  6FNORED   5NA       4NA       3NA       2NA       1SEC01    0SEC00*/
/*         0       1         0         0         0         0         1         0     */   
const unsigned char NVOPT_INIT @0x0000FFBF = 0x42;




word ad_image, ad_ref;
//word scan[128]; 

#pragma DATA_SEG __SHORT_SEG MY_ZEROPAGE

volatile byte AD_ready_flag; //flag that indicates AD ready
volatile word ADvalue; //value of A to D channel read
byte pct;  //keeps track of clock pulse count
byte ctrate; //time period between samples in 100 ms
byte exposure;  //exposure delay
word lightstrength; //strength of light 0-255

word maxpixel,maxpixelfinal;  //indicate maximum light value (for detecting over-exposure)
byte count;  //keeps track of countdown in timing interrupt for triggering a sample
word lastpixel;
word maxstep,minstep;  //tracks largest step changes in line scan
byte maxsteploc,minsteploc;  //tracks location of largest step change in linescan (pixel)
byte width;  //width of the filament in pixels
byte timer;  //counts up

#pragma DATA_SEG DEFAULT

fixed x0,x1,x2,x3,ct;
fixed a1,b1,c1,a2,b2,c2,m1,m2; //sub pixel quadratic interpolation variables
fixed widthsubpixel,widthsubpixellp; 
byte widthsubpixelstore;
byte widthfilterparam;
byte keypress;  //used to track previous keypress

fixed width_calib_value;  //calibration value for the width

byte ad_ref_store;
fixed ad_ref_lp;

//Variables also stored in FLASH start here
volatile byte param1;  //currently unused parameter
volatile byte measflag; //indicates type of measurement, 0=ratiometric, >0=absolute
volatile fixed width_calib_fact;   //calibration factor for the width
//End of variables stored in FLASH

fixed width_calib;  //calibrated width

byte capture_minstep,capture_maxstep; //indicates max delta found, so capture buffers

#define LSA_SI PTBD_PTBD1
#define LSA_CLK PTBD_PTBD0
 
void CycleAD(byte AD_chan);


 

//Flash routines ****************************************************************

#define FLASHBLOCK 4  //size selected for the block of flash to write
#define FLASHSIZE 512 //size selected for the contiguous flash area

//Flash defines
// place destination Flash data into the proper segment, see also project.prm 

#pragma CONST_SEG DEFAULT_EEPROM
extern volatile unsigned char flashloc @0xE000;
volatile unsigned char flashloc;
#pragma CONST_SEG DEFAULT





//writes to the next available 4 byte block of flash
byte flashwrite(unsigned char * flashstart, unsigned char * ramloc)
  {
  word i;
  byte flashres;
  i=0;
  
  if(ramloc[0]==255)     //check that the first byte does not have 255, the flag for empty FLASH 
    return(255);         //return an error if so
    
    
    
  while ((flashstart[i] !=255) && (i<FLASHSIZE))
    {
    i=i+FLASHBLOCK;
    }
    
  if(flashstart[i]!=255)  //If we can't find any erased space, we know we need to erase the first sector
    {
    i=0; //reset to first block
    FlashErase(flashstart); //erase first sector
    }
  if(flashstart[i]==255) //check that we have an erased block
    { 
    flashres=FlashProgBurst(&flashstart[i], ramloc, FLASHBLOCK); //write the data from RAM to the FLASH returns 255 if in error, 0 if OK
    i=i+FLASHBLOCK;  //advance index to next block
    
    if(i>FLASHSIZE) //check if at end of flash area
      {
      FlashErase(flashstart); //erase first sector
      }
      
    else if(flashstart[i]!=255)  //check if the next block is erased as well.  If not it is because we are crossing a sector 
      {
      FlashErase(&flashstart[i]); //erase next sector
      }
    }
    else
    flashres=255; //return an error flag
  



  return(flashres);
  }

//Reads current data from flash and moves to RAM
byte flashread(unsigned char * flashstart, unsigned char * ramloc)
  {
  word i,j;
  i=0;
  j=0;   
      
  while ((flashstart[i] !=255) && (i<FLASHSIZE))
    {
    i=i+FLASHBLOCK;
    }
  
  if(i==0)  //check if first block still erased
    return(255);  //no data to transfer
    
    i=i-FLASHBLOCK; //if out of loop because i<FLASHSIZE, reset to last block in area.  if out of loop because flashstart[i] !=255 reset to last block.
  
  while (j<FLASHBLOCK)
    {
    ramloc[j]=flashstart[i];
    j=j+1;
    i=i+1;
    }
    

  return(0);
  }






void clearlin(void){
	
	byte i;
	
	LSA_SI=1;
	LSA_CLK=1;
	LSA_SI=0;
	LSA_CLK=0;
	
	
			
	for(i=127;i>0;i--)
		{		
		LSA_CLK=1;   
		LSA_CLK=0;
		}


}

void main(void) {
	
//initialize the crystal pins to high outputs
	
	//PTBD_PTBD6 = 1;
	//PTBD_PTBD7 = 0;
	
	//PTBDD_PTBDD6=1;
	//PTBDD_PTBDD7=1;
	
	
  MCU_init();	/* call Device Initialization */
  /* include your code here */

  //for writing to FLASH
    FCDIV=vFCDIV; //set FCLK divider (FCLK=200KHz)
  
  
  //asm_main();		/* call the assembly function */


	  
//	if(PTBD_PTBD2)
//		PTBD_PTBD3=1;
//	else
//		PTBD_PTBD3=0;
	  
	  
 
  
  

    
    //Set drive strength for PWM pins to high
    PTBDS_PTBDS5=1;
    PTADS_PTADS0=1;
    
    
   
   // timedelay(10);
    count=0; 
    ctrate=2;  //time between samples in .1 seconds
    widthfilterparam=5;  //low pass filter parameter
    //count=ctrate;
    
    exposure=10;  //set exposure time
    lightstrength=560;  //set strength of LED
   
    timer=0;
    
    
	  if(flashread(&flashloc, &param1)>0)  //read the flash data to RAM, if available
	    {
		  //no flash to read, so first time run after programming
		  //Initialize the RAM that will be stored in the flash
		  param1=0;  //unused parameter.  Can't be 255 since its used for detecting erased flash too
		  measflag=0;  //indicate ratiometric
		  width_calib_fact=250;   //set to .979 in fixed point
	    }
	  
	  
    
    
    //calibrate using a drill rod of .062 in diameter (1.57 mm)
    width_calib_value=6329;     //set to (1.57 mm)*(15.748 pixels/mm)*(256 fp)=6329
    
    for(;;)
    {
  	  
  	  
  	  if (count==0)  //countdown expired so run sample
  	  {
  		  
  		  timer=timer+1;
  		  
  		  if(keypress==0 || keypress>250)
  			  {
  			  if(measflag==0 || (timer & 64))  //in ratiometric mode or pulse light
  				  PTBD_PTBD3=1;  //turn on display led
  			  
  			  }
  			  
  		  
  		  maxstep=0;
  		  minstep=0;
  		  maxsteploc=255;
  		  minsteploc=255;
  		  maxpixel=0;
 	  
  		  TPM1C0V=lightstrength;  //set the pwm for the LED
  		  
  		  //clear the sub-pixel buffers
  		  x0=0;
  		  x1=0;
  		  x2=0;
  		  x3=0;
  		  ct=-2;  //index to count samples  need to load buffer for 2 steps to subtract x2-x1
  		  
  		  
 		  
  		  clearlin();	//clear array
  		  timedelay(exposure);  //wait for exposure time and 	delay at least 20uS to allow sampling capacitor to charge
  		  LSA_SI=1;
  		  LSA_CLK=1;
  		  LSA_SI=0;
  		  //timedelay(5);
  		  for(pct=0;pct<=127;pct++)
  		  {
  			  
  			  //asm{
  			 //			  nop
  			 //		  }
  			  
  			  LSA_CLK=1;
  			  
  			  /*Trigger a measurement of the A to D converter*/
  			  CycleAD(1);  //read channel 1 for image sensor
  			  
  			  while(AD_ready_flag==0);  //wait til conversion
  			  
  			   /*A to D Conversion*/
  				  /*Get the A to D measurements*/
  				  ad_image=ADvalue;
  				  AD_ready_flag=0;
  				  
	 			  LSA_CLK=0; 	//make clock go low 			  
  	 			  
  	  			  if(ad_image>maxpixel)
  	  				  maxpixel=ad_image;
  	  			  
  	 		  // update the sliding buffers
  	 		  x3=x2;  
  	 		  x2=x1;
  	 		  x1=x0;
  	 		  x0=ad_image;
  	 		  ct=ct+1;
  	 		  
  			 // scan[pct]=ad_image;  //transfer the pixel to the array
  			  
  			 //simple pixel edge detection algorithm
  			  
  			  //check if detecting an edge
  			  if(ct>1 && ct<126) //ensure buffer is full
  			  {
  				if(x1<x2)  
  					{
  					if(minstep<x2-x1)
  					
  					{
  						minstep=x2-x1;
  						minsteploc=ct;
  						c1=x1-x0;
  						b1=x2-x1;
  						a1=x3-x2;
  						
  					}
  					}
  				else
  				{
  					if(maxstep<x1-x2)
  					{
  						maxstep=x1-x2;
  						maxsteploc=ct;
  						c2=x1-x0;
  						b2=x2-x1;
  						a2=x3-x2;
  						
  					}
  					
  				}
  			  }
  			  lastpixel=ad_image;
 


  			  
  			  
  		  }
  		  
  		  //The 129th clock pulse
  		  LSA_CLK=1;
  		  //timedelay(5);
  		  
  		 // asm{
  		//	  nop
  		//  }
  		  
  		  
  		  LSA_CLK=0;
  		  //timedelay(5);
  		  
  		  maxpixelfinal=maxpixel;
  		  
  		  if(minstep>16 && maxstep>16)  //check for significant threshold
  		  {
  			  width=maxsteploc-minsteploc;
  			  
  		  }
  		  else
  			  width=0;
  		if(width>103)  //check for width overflow or out of range (15.7pixels per mm, 65535/635=103)
  			width=0;
  		
  		
		  //sub-pixel edge detection using interpolation
		  
		  m1=fdiv((a1-c1),(a1+c1-(b1<<1)))>>1;
		  m2=fdiv((a2-c2),(a2+c2-(b2<<1)))>>1;
  		 	 
		
		  if(width>15)		//check for a measurement > 1mm  otherwise treat as noise
			  widthsubpixel=(width<<8)+m2-m1; 
		  else
			  widthsubpixel=0;
		  
		  widthsubpixellp=lowpass(widthsubpixel,widthsubpixellp,widthfilterparam,&widthsubpixelstore);
		  
		  width_calib=fmulc(widthsubpixellp,width_calib_fact);
  		
  	
		  /*Trigger a measurement of the A to D converter*/
			  CycleAD(27);  //read channel 27 for bandgap ref
			  
			  while(AD_ready_flag==0);  //wait til conversion
			  
			   /*A to D Conversion*/
				  /*Get the A to D measurements*/
				  ad_ref=ADvalue;
				  AD_ready_flag=0;
		  
				  ad_ref_lp=lowpass(ad_ref,ad_ref_lp,widthfilterparam,&ad_ref_store);
		  
		  
		  
		  if (measflag==0)
			  {
  			//  3mm is 47.25 pixels.  scale that to 614 fp output (3.00 volts)
  			//  614/256/47.25=.050760  (12.99456 fp)
		  TPM1C1V=fmulc(width_calib,13);  //set the pwm for the output voltage ratiometric
			  }
		  else
			  {
			  //how to adjust output to absolute volts
			 		  //calculate supply voltage from measurement of band gap Vss= (1.20v)*1024/abandgap	
			 		  //output percent = (pout* abandgap)/(1.2*1024);
			 		  //        =(widthcalib * abandgap)/(78.75*248)*1024
			 		  //        =(widthcalib * abandgap)*.052858433    .052858433 is 13.5317 in FP
			  	  	  // 433/32 = 13.53125
			  
			  TPM1C1V=fmulc(fmulc(width_calib,ad_ref_lp),433) >> 5;  //set the pwm for the output voltage absolute
			  }
		  //calibrate lightstrength
		  if(maxpixel<1000 && lightstrength<1023)
			  lightstrength=lightstrength+1;
		  else if(lightstrength>0)
			  lightstrength=lightstrength-1;
		  
  		//calibrate width
		  if(PTBD_PTBD2==0)   //calibration button is pressed
		  {
			  if(keypress<255)
				  keypress=keypress+1;
			  if(keypress==250)  //wait until key held for 2 sec
				  {
				  width_calib_fact=fdiv(width_calib_value,widthsubpixellp);
				  flashwrite(&flashloc, &param1);  //send location of start of FLASH area and start of RAM
				  }
			  
			  
			  
		  }
		  else
			  {
			  if(keypress>5 && keypress<50)  //short press is released
				  {
				  if(measflag==0)
					  measflag=1;
				  else
					  measflag=0;
				  flashwrite(&flashloc, &param1);  //send location of start of FLASH area and start of RAM
				  }
				  
				  
			  keypress=0;
			  }
		  
		PTBD_PTBD3=0;  //turn off display led
  		count=ctrate;  //reset count  //debug run open loop
  	 }
    }
  
  
  
  for(;;) {
    /* __RESET_WATCHDOG(); By default COP is disabled with device init. When enabling, also reset the watchdog. */
  } /* loop forever */
  /* please make sure that you never leave main */
}
